@@ -4,8 +4,8 @@ vcr__env <- new.env()
 #'
 #' @export
 #' @inheritParams use_cassette
-#' @param ignore_cassettes (logical) turn \pkg{vcr} off and ignore cassette
-#' insertions (so that no error is raised). Default: `FALSE`
+#' @inheritSection use_cassette Cassette options
+#' @inheritSection check_cassette_names Cassette names
 #' @seealso [use_cassette()], [eject_cassette()]
 #' @return an object of class `Cassette`
 #' @examples \dontrun{
@@ -26,14 +26,19 @@ vcr__env <- new.env()
 #' # cleanup
 #' unlink(file.path(tempdir(), "leo5.yml"))
 #' }
-insert_cassette <- function(name, record="once",
-  match_requests_on = c('method', 'uri'),
-  update_content_length_header=FALSE,
-  allow_playback_repeats=FALSE, serialize_with="yaml",
-  persist_with="FileSystem",
-  preserve_exact_body_bytes=FALSE, ignore_cassettes = FALSE, 
-  re_record_interval = NULL, clean_outdated_http_interactions = FALSE) {
+insert_cassette <- function(name,
+  record = NULL,
+  match_requests_on = NULL,
+  update_content_length_header = FALSE,
+  allow_playback_repeats = FALSE,
+  serialize_with = NULL,
+  persist_with = NULL,
+  preserve_exact_body_bytes = NULL,
+  re_record_interval = NULL,
+  clean_outdated_http_interactions = NULL) {
 
+  check_cassette_name(name)
+  vcr_env_handle()
   if (turned_on()) {
     if ( any( name %in% names(cassettes_session()) ) ) {
       stop(sprintf("There is already a cassette with the same name: %s", name),
@@ -49,20 +54,24 @@ insert_cassette <- function(name, record="once",
     vcr__env$current_cassette <- name
 
     # make cassette
-    tmp <- Cassette$new(
-      name = name, record = record, match_requests_on = match_requests_on,
-      re_record_interval = re_record_interval, 
-      clean_outdated_http_interactions = clean_outdated_http_interactions,
-      tag = NULL, tags = NULL,
+    tmp <- Cassette$new(name,
+      record = record %||% vcr_c$record,
+      match_requests_on = match_requests_on %||% vcr_c$match_requests_on,
       update_content_length_header = update_content_length_header,
-      decode_compressed_response = NULL,
       allow_playback_repeats = allow_playback_repeats,
+      serialize_with = serialize_with %||% vcr_c$serialize_with,
+      persist_with = persist_with %||% vcr_c$persist_with,
+      preserve_exact_body_bytes = preserve_exact_body_bytes %||% vcr_c$preserve_exact_body_bytes,
+      re_record_interval = re_record_interval %||% vcr_c$re_record_interval,
+      clean_outdated_http_interactions = clean_outdated_http_interactions %||% vcr_c$clean_outdated_http_interactions,
+      tag = NULL,
+      tags = NULL,
+      decode_compressed_response = NULL,
       allow_unused_http_interactions = NULL,
-      exclusive = NULL,
-      serialize_with = serialize_with, persist_with = persist_with,
-      preserve_exact_body_bytes = preserve_exact_body_bytes)
+      exclusive = NULL
+    )
     return(tmp)
-  } else if (ignore_cassettes) {
+  } else if (!light_switch$ignore_cassettes) {
     message <- "vcr is turned off.  You must turn it on before you can insert a cassette.
       Or you can set ignore_cassettes=TRUE option to completely ignore cassette insertions."
     stop(message)
